# This file is part of the graph-includes package
#
# (c) 2005 Yann Dirson <ydirson@altern.org>
# Distributed under version 2 of the GNU GPL.

package graphincludes::project::default;
use graphincludes::project;
our @ISA = qw(graphincludes::project);

sub new {
  my $class = shift;
  my $self;

  $self  = $class->SUPER::new(@_);

  bless ($self, $class);

  $self->{IGNOREDDEPS} = $self->ignored_deps();
  $self->{IGNOREDEDGES} = {};	# to be computed in getdeps

  return $self;
}

sub filelabel {
  my $self = shift;
  my ($file,$level) = @_;
  $level = $main::minshow unless defined $level;

  $file =~ s/^$self->{PFXSTRIP}// if defined $self->{PFXSTRIP};
  if ($level == 0) {
    return $file;
  } elsif ($level == 1) {
    $file =~ s/\.[^.]*$//;
    return $file;
  } elsif ($level == 2) {
    if ($file =~ m!(.*)/.*!) {
      return $1;
    } else {
      return '<' . $self->filelabel($file, $level - 1) . '>';
    }
  }
  return undef;
}

sub defaultcolors {
  return ();
}

sub ignored_deps {
  return {};
}

sub special_edge {
  my $self = shift;
  my ($src, $dst) = @_;

  my $lbl = $self->{IGNOREDEDGES}->{$src}->{$dst};

  if (defined $lbl) {
    return "color=gray,constraint=false,label=\"$lbl\"";
  } else {
    return $self->SUPER::special_edge($src,$dst);
  }
}

sub getdeps {
  my $self = shift;
  my %deps;

  sub fullpath {
    my ($dstpath, $strip, $srcpath) = @_;
    join ('/', @$srcpath[0..($#$srcpath-$strip)], @$dstpath);
  }

  while (<>) {
    if (m/^\s*#\s*include\s*"(.*)"/) {
      my @srcpath = split (m|/|, $ARGV); # can we avoid to compute this for each dep ?
      my @dstpath = split (m|/|, $1);
      # count number of leading "../" in the #include reference
      my $strip = 1;		# we'll have to strip at least the basename from srcpath
      while ($dstpath[0] eq '..') {
	$strip++; shift @dstpath;
      }
      # find the file, assuming lookup order should be -I. -I.. -I../.. etc...
      my $dstfile;
      while (scalar(@srcpath) > $strip and
		   $dstfile = fullpath(\@dstpath,$strip,\@srcpath) and
		   !grep { $_ eq $dstfile } @{$self->{FILES}}) {
	$strip++;
      }
      #
      if (grep { $_ eq $dstfile } @{$self->{FILES}}) { # FIXME: duplicated expensive test
	my $orignode = $self->{NODEIDS}->{$self->filelabel($ARGV)};
	my $destnode = $self->{NODEIDS}->{$self->filelabel($dstfile)};
	if (defined $self->{IGNOREDDEPS}->{$ARGV}->{$dstfile}) {
	  print STDERR "ignoring $ARGV -> $dstfile\n" if $main::debug;
	  $self->{IGNOREDEDGES}->{$orignode}->{$destnode} =
	    $self->{IGNOREDDEPS}->{$ARGV}->{$dstfile};
	}
	push (@{$deps{$orignode}}, $destnode);
      } else {
	print STDERR "ERROR: unknown file $dstfile referenced by $ARGV\n" if $main::verbose >= 2;
      }
    } elsif (m/^\s*#\s*include\s*<(.*)>/) {
      print STDERR "Ignoring <$1> for now ($ARGV)\n" if $main::verbose >= 2;
    }
  }
  return \%deps;
}

1;
